"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createDocument = void 0;
const syntax_tree_1 = require("./syntax-tree");
const compile_bindings_1 = require("./compile-bindings");
const diagnostic_1 = require("../diagnostic");
const utils_1 = __importDefault(require("../utils"));
// Build binding tree
function createDocument(filePath, tokens, reporting) {
    var _a, _b, _c;
    const domNodeStack = []; // Keeps track of unbalanced documents
    const imports = []; // Imported symbols
    const astContextStack = []; // The current context for all bindings in the tree
    const allBindings = new Set();
    function overrideViewmodel(vmType) {
        const _ = astContextStack.pop();
        const parent = astContextStack.pop();
        if (parent) {
            astContextStack.push(parent);
            const newNode = new syntax_tree_1.TypeNode(parent, vmType);
            parent.childNodes.push(newNode);
            astContextStack.push(newNode);
        }
        else {
            astContextStack.push(new syntax_tree_1.TypeNode(parent, vmType));
        }
    }
    for (const node of tokens) {
        if (node instanceof syntax_tree_1.DiagNode) {
            if (node.enable) {
                if (node.keys.length)
                    reporting.enableDiagnostics(node.keys);
                else {
                    reporting.enableAllDiagnostics();
                }
            }
            else {
                if (node.keys.length)
                    reporting.disableDiagnostics(node.keys);
                else
                    reporting.disableAllDiagnostics();
            }
            // TODO: Only disable diagnostics inside of current element block.
            continue;
        }
        if (node instanceof syntax_tree_1.ImportNode) {
            imports.push(node);
            continue;
        }
        if (node instanceof syntax_tree_1.ChildContextNode) {
            overrideViewmodel(node);
            continue;
        }
        switch (node.nodeType) {
            case syntax_tree_1.NodeType.Start: {
                let currentContext = astContextStack[astContextStack.length - 1];
                if (!currentContext)
                    throw new diagnostic_1.Diagnostic(filePath, 'no-viewmodel-reference', Object.assign({}, node.loc));
                if ((_a = node.bindings) === null || _a === void 0 ? void 0 : _a.length) {
                    // There could be multiple data-binds on one element. Parse them all.
                    // TODO: If there are multiple binding properties on one row, emit a warning if it controls descendants (unclear semantics of multiple bindings?)
                    const bindings = utils_1.default.flat(node.bindings.map(bindingData => compile_bindings_1.parseBindingExpression(filePath, reporting, bindingData.bindingText, bindingData.location)));
                    if (bindings.length) {
                        for (const b of bindings)
                            allBindings.add(b.bindingHandler.name);
                        const bindingNode = new syntax_tree_1.BindingNode(currentContext, bindings);
                        currentContext.childNodes.push(bindingNode);
                        currentContext = bindingNode;
                    }
                }
                astContextStack.push(currentContext);
                domNodeStack.push(node);
                break;
            }
            case syntax_tree_1.NodeType.End: {
                astContextStack.pop();
                const lastNode = domNodeStack.pop();
                if ((lastNode === null || lastNode === void 0 ? void 0 : lastNode.key) !== node.key)
                    throw new diagnostic_1.Diagnostic(filePath, diagnostic_1.diagnostics['unbalanced-start-end-tags'], lastNode === null || lastNode === void 0 ? void 0 : lastNode.loc);
                break;
            }
            case syntax_tree_1.NodeType.Empty: {
                if ((_b = node.bindings) === null || _b === void 0 ? void 0 : _b.length) {
                    // TODO: parse all binding strings
                    const bindings = utils_1.default.flat(node.bindings.map(bindingData => compile_bindings_1.parseBindingExpression(filePath, reporting, bindingData.bindingText, bindingData.location)));
                    if (bindings.length) {
                        for (const b of bindings)
                            allBindings.add(b.bindingHandler.name);
                        const currentContext = astContextStack[astContextStack.length - 1];
                        if (!currentContext)
                            throw new diagnostic_1.Diagnostic(filePath, 'no-viewmodel-reference', Object.assign({}, node.loc));
                        const bindingNode = new syntax_tree_1.BindingNode(currentContext, bindings);
                        currentContext.childNodes.push(bindingNode);
                    }
                }
                break;
            }
        }
    }
    if (domNodeStack.length)
        reporting.addDiagnostic(new diagnostic_1.Diagnostic(filePath, diagnostic_1.diagnostics['unbalanced-start-end-tags'], (_c = domNodeStack.pop()) === null || _c === void 0 ? void 0 : _c.loc));
    return new syntax_tree_1.Document(filePath, astContextStack[0], imports, [...allBindings]);
}
exports.createDocument = createDocument;
//# sourceMappingURL=document-builder.js.map