import { Location } from './location';
export declare enum NodeType {
    Start = 0,
    End = 1,
    Empty = 2
}
export declare class BindingData {
    location: Location;
    bindingText: string;
    constructor(location: Location, bindingText: string);
}
/**
 * XML and HTML documents are parsed to a series of nodes (non hierarchical).
 */
export declare class Node {
    loc: Location;
    key: string;
    nodeType: NodeType;
    constructor(loc: Location, key: string, nodeType: NodeType);
    bindings: BindingData[] | undefined;
}
export declare class TypeReferenceNode extends Node {
    ref: IdentifierNode<string>;
    isType: boolean;
    constructor(location: Location, ref: IdentifierNode<string>, isType: boolean);
}
export declare class ChildContextNode extends Node {
    type: TypeReferenceNode;
    constructor(location: Location, type: TypeReferenceNode);
}
export declare class NamedContextNode extends Node {
    ident: IdentifierNode<string>;
    constructor(location: Location, ident: IdentifierNode<string>);
}
export declare class ContextAssignmentNode extends Node {
    ident: IdentifierNode<string>;
    x?: string | undefined;
    constructor(location: Location, ident: IdentifierNode<string>, x?: string | undefined);
}
/** Lint Node identifying a viewmodel module to use during type checking */
export declare class ImportNode extends Node {
    importSymbols: {
        name: IdentifierNode<string>;
        alias: IdentifierNode<string>;
    }[];
    modulePath: IdentifierNode<string>;
    constructor(location: Location, importSymbols: {
        name: IdentifierNode<string>;
        alias: IdentifierNode<string>;
    }[], modulePath: IdentifierNode<string>);
}
export interface BindingHandlerImport {
    isTypeof: boolean;
    name: IdentifierNode<string>;
    alias: IdentifierNode<string>;
    index: number;
}
/** Enable/Disable flags for single or multiple diagnostics. (ko-view-lint and typescript error codes) */
export declare class DiagNode extends Node {
    keys: string[];
    enable: boolean;
    constructor(loc: Location, keys: string[], enable: boolean);
}
export declare class IdentifierNode<T> {
    value: T;
    location: Location;
    constructor(value: T, location: Location);
}
export declare class BindingName {
    name: string;
    loc: Location;
    constructor(name: string, loc: Location);
}
export declare class BindingExpression {
    text: string;
    loc: Location;
    constructor(text: string, loc: Location);
}
export declare class Document {
    viewFilePath: string;
    rootNode: TypeNode;
    imports: ImportNode[];
    bindingNames: string[];
    rootContext?: BindingContext;
    /**
     * The hierarchical representation of all the bindings in the view.
     * @param viewFilePath file path to the view to compile
     * @param rootBinding Top-level node representing the view. All data-bindings in the view are located as descendants to the root node.
     * @param imports All viewmodels associated to the view, usually only one.
     * @param bindingHandlerReferences All the view's used bindinghandlers except for the ones that are already globally available.
     */
    constructor(viewFilePath: string, rootNode: TypeNode, imports: ImportNode[], bindingNames: string[]);
}
export declare class BindingContext {
    parent?: BindingContext | undefined;
    typeAssertion?: string | undefined;
    isType: boolean;
    id: string;
    viewModelReference: ImportNode | undefined;
    childBindings: Binding[];
    childContexts: BindingContext[];
    childBindingQueue: BindingNode[];
    private constructor();
    static createRoot(type: TypeNode): BindingContext;
    copyContext(): BindingContext;
    createChildContext(): BindingContext;
}
export declare abstract class AstNode {
    parent?: AstNode | undefined;
    childNodes: AstNode[];
    childContext: BindingContext | undefined;
    protected constructor(parent?: AstNode | undefined);
    getContext(): BindingContext;
    getParentContext(): BindingContext;
}
export declare class TypeNode extends AstNode {
    vm?: ChildContextNode | undefined;
    constructor(parent?: AstNode, vm?: ChildContextNode | undefined);
}
export declare class BindingNode extends AstNode {
    bindings: Binding[];
    constructor(parent: AstNode, bindings: Binding[]);
}
export declare class Binding {
    bindingHandler: BindingName;
    expression: BindingExpression;
    private static identifierIndex;
    identifierName: string;
    diagFlags: DiagNode | undefined;
    /**
     *
     * @param bindingHandler key
     * @param valueRaw value
     * @param parentBinding
     */
    constructor(bindingHandler: BindingName, expression: BindingExpression);
}
//# sourceMappingURL=syntax-tree.d.ts.map