"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Binding = exports.BindingNode = exports.TypeNode = exports.AstNode = exports.BindingContext = exports.Document = exports.BindingExpression = exports.BindingName = exports.IdentifierNode = exports.DiagNode = exports.ImportNode = exports.ContextAssignmentNode = exports.NamedContextNode = exports.ChildContextNode = exports.TypeReferenceNode = exports.Node = exports.BindingData = exports.NodeType = void 0;
const utils_1 = __importDefault(require("../utils"));
///
/// AST Node classes
///
var NodeType;
(function (NodeType) {
    NodeType[NodeType["Start"] = 0] = "Start";
    NodeType[NodeType["End"] = 1] = "End";
    NodeType[NodeType["Empty"] = 2] = "Empty";
})(NodeType = exports.NodeType || (exports.NodeType = {}));
class BindingData {
    constructor(location, bindingText) {
        this.location = location;
        this.bindingText = bindingText;
    }
}
exports.BindingData = BindingData;
///
/// Shared classes for DOM and AST
/// TODO: Clean up and separate
///
/**
 * XML and HTML documents are parsed to a series of nodes (non hierarchical).
 */
class Node {
    constructor(loc, key, nodeType) {
        this.loc = loc;
        this.key = key;
        this.nodeType = nodeType;
    }
}
exports.Node = Node;
class TypeReferenceNode extends Node {
    constructor(location, ref, isType) {
        super(location, '', NodeType.Empty);
        this.ref = ref;
        this.isType = isType;
    }
}
exports.TypeReferenceNode = TypeReferenceNode;
class ChildContextNode extends Node {
    constructor(location, type) {
        super(location, '', NodeType.Empty);
        this.type = type;
    }
}
exports.ChildContextNode = ChildContextNode;
class NamedContextNode extends Node {
    constructor(location, ident) {
        super(location, '', NodeType.Empty);
        this.ident = ident;
    }
}
exports.NamedContextNode = NamedContextNode;
class ContextAssignmentNode extends Node {
    constructor(location, ident, x) {
        super(location, '', NodeType.Empty);
        this.ident = ident;
        this.x = x;
    }
}
exports.ContextAssignmentNode = ContextAssignmentNode;
/** Lint Node identifying a viewmodel module to use during type checking */
class ImportNode extends Node {
    constructor(location, importSymbols, modulePath) {
        super(location, '', NodeType.Empty);
        this.importSymbols = importSymbols;
        this.modulePath = modulePath;
    }
}
exports.ImportNode = ImportNode;
/** Enable/Disable flags for single or multiple diagnostics. (ko-view-lint and typescript error codes) */
class DiagNode extends Node {
    constructor(loc, keys, enable) {
        super(loc, '', NodeType.Empty);
        this.keys = keys;
        this.enable = enable;
    }
}
exports.DiagNode = DiagNode;
class IdentifierNode {
    constructor(value, location) {
        this.value = value;
        this.location = location;
    }
}
exports.IdentifierNode = IdentifierNode;
///
/// Document Model Classes
///
class BindingName {
    constructor(name, loc) {
        this.name = name;
        this.loc = loc;
    }
}
exports.BindingName = BindingName;
class BindingExpression {
    constructor(text, loc) {
        this.text = text;
        this.loc = loc;
    }
}
exports.BindingExpression = BindingExpression;
class Document {
    /**
     * The hierarchical representation of all the bindings in the view.
     * @param viewFilePath file path to the view to compile
     * @param rootBinding Top-level node representing the view. All data-bindings in the view are located as descendants to the root node.
     * @param imports All viewmodels associated to the view, usually only one.
     * @param bindingHandlerReferences All the view's used bindinghandlers except for the ones that are already globally available.
     */
    constructor(viewFilePath, rootNode, imports, bindingNames) {
        this.viewFilePath = viewFilePath;
        this.rootNode = rootNode;
        this.imports = imports;
        this.bindingNames = bindingNames;
        this.viewFilePath = utils_1.default.canonicalPath(viewFilePath);
    }
}
exports.Document = Document;
let contextId = 0;
class BindingContext {
    constructor(parent, typeAssertion, isType = true) {
        this.parent = parent;
        this.typeAssertion = typeAssertion;
        this.isType = isType;
        this.childBindings = []; // All validated child bindings
        this.childContexts = [];
        this.childBindingQueue = []; // Unprocessed binding nodes
        contextId++;
        this.id = typeAssertion ?
            `context_${contextId}_${typeAssertion}` :
            `context_${contextId}`;
    }
    static createRoot(type) {
        var _a, _b;
        return new BindingContext(undefined, (_a = type.vm) === null || _a === void 0 ? void 0 : _a.type.ref.value, (_b = type.vm) === null || _b === void 0 ? void 0 : _b.type.isType);
    }
    copyContext() {
        var _a;
        const context = new BindingContext(this.parent, this.typeAssertion);
        context.viewModelReference = this.viewModelReference;
        (_a = this.parent) === null || _a === void 0 ? void 0 : _a.childContexts.push(context);
        return context;
    }
    createChildContext() {
        const childContext = new BindingContext(this);
        childContext.viewModelReference = this.viewModelReference;
        this.childContexts.push(childContext);
        return childContext;
    }
}
exports.BindingContext = BindingContext;
class AstNode {
    constructor(parent) {
        this.parent = parent;
        this.childNodes = [];
    }
    getContext() {
        if (this.childContext)
            return this.childContext;
        if (this.parent)
            return this.parent.getContext();
        throw new Error('No top-level context was defined');
    }
    getParentContext() {
        const parent = this.parent;
        if (!parent)
            throw new Error('Missing expected parent node.');
        return parent.getContext();
    }
}
exports.AstNode = AstNode;
class TypeNode extends AstNode {
    constructor(parent, vm) {
        super(parent);
        this.vm = vm;
    }
}
exports.TypeNode = TypeNode;
class BindingNode extends AstNode {
    constructor(parent, bindings) {
        super(parent);
        this.bindings = bindings;
    }
}
exports.BindingNode = BindingNode;
class Binding {
    /**
     *
     * @param bindingHandler key
     * @param valueRaw value
     * @param parentBinding
     */
    constructor(bindingHandler, expression) {
        this.bindingHandler = bindingHandler;
        this.expression = expression;
        this.identifierName = 'binding_' + (Binding.identifierIndex++).toString();
    }
}
exports.Binding = Binding;
Binding.identifierIndex = 0;
//# sourceMappingURL=syntax-tree.js.map